function clist = etsi_cablesADSL(clist)
%% ===========================================================================
%etsi_cablesADSL(clist) - Sets up the cable models for ETSI
%
% Parameter:    clist          Structure describing the cables
% Returns:      clist          Structure describing the cables
%
% Example(s):
%   ex.clist = etsi_cablesADSL(ex.clist);              Add to existing clist
%   cableparam = getList(ex.clist,'ETSI-ADSL-PE032');  Get a PE 0.32 ADSL cable
%
%   ex.clist = etsi_cablesADSL([]);                    Create a new clist
%   cableparam = getList(ex.clist,'ETSI-ADSL-PE05');   Get a PE 0.5 ADSL cable
%
% Reference:
%   ETSI STC TM6 RTS/TM-06025 rev1a 2001-09
%   ETSI STC TM6 PD 980p10a5 "Laboratory Performance tests for xDSL
%   systems" (feb 5, 2001)
%   Draft ITU-T Recommendation G.996.1 "Test procedures for DSL tranceivers"
%   ETSI TM6 021t32, Torino, Italy, 4th-8th Feb., 2002.
%   ETSI STC TM6 DTS/TM-06018 (Draft m00p10a5NC) 2001-03
%   ETSI STC TM6 RTS/TM-06025 (ETSI TS 101388 V1.3.1; 2002-03)    
%% ===========================================================================

%% ===========================================================================
% Copyright (C):                                        
%       2000-2007 by Forschungszentrum Telekommunikation Wien, Austria;
%                                                         All rights reserved.
% Project       : FTW's xDSLsimu
% Author(s)     : Tomas Nordstrom (Tomas.Nordstrom@FTW.at)
%
% CVS:       $Id: etsi_cablesADSL.m 685 2007-01-16 14:46:11Z tono $
%% ===========================================================================
% Change History
%      2000-07-19 (ToNo) Changed so that r0s can be zero for BT models
%      2001-07-04 (ToNo) Added cable models used in ADSL simulations
%      2002-01-11 (ToNo) Added ETSI ADSL as param. models from ETSI 981t08a1
%      2002-02-01 (ToNo) Updated  the original ETSI ADSL parameters
%      2002-08-23 (ToNo) Split up according to technology
%      2003-03-28 (ToNo) Updated list according to (TS 101388 V1.3.1; 2002-03)
%% ===========================================================================

%% ===========================================================================
% ETSI - ADSL Normative Cable Parameters
% From ETSI STC TM6 RTS/TM-06025 (ETSI TS 101388 V1.3.1; 2002-03)
%

cable=[];
cable.name='ETSI-ADSL-PE032';
cable.model='BT';

Ro     = 409;
ac     = 0.3822;
Lo     = 607.64e-6;
Linf   = 500.00e-6;
Fm     = 608.77e3;
Nb     = 5.2464;
Cp     = 40e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = ac;
cable.param.as  = 0;
cable.param.l0  = Lo;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);


cable=[];
cable.name='ETSI-ADSL-PE04';
cable.model='BT';

Ro     = 280;
ac     = 0.0969;
Lo     = 587.13e-6;
Linf   = 427.12e-6;
Fm     = 739.05e3;
Nb     = 1.3952;
Cp     = 50e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = ac;
cable.param.as  = 0;
cable.param.l0  = Lo;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE05';
cable.model='BT';

Ro     = 179;
ac     = 0.0561;
Lo     = 673.57e-6;
Linf   = 544.25e-6;
Fm     = 580.92e3;
Nb     = 1.3013;
Cp     = 50e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = ac;
cable.param.as  = 0;
cable.param.l0  = Lo;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);


cable=[];
cable.name='ETSI-ADSL-PE063';
cable.model='BT';

Ro     = 113;
ac     = 0.0256;
Lo     = 699.26e-6;
Linf   = 477.42e-6;
Fm     = 265.70e3;
Nb     = 1.0978;
Cp     = 45e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = ac;
cable.param.as  = 0;
cable.param.l0  = Lo;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE09';
cable.model='BT';

Ro     = 55;
ac     = 0.0094;
Lo     = 750.79e-6;
Linf   = 520.45e-6;
Fm     = 124.04e3;
Nb     = 0.9605;
Cp     = 40e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = ac;
cable.param.as  = 0;
cable.param.l0  = Lo;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);

    
%%============================================================================
% ETSI - ADSL Informative Cable Parameters
% From ETSI STC TM6 RTS/TM-06025 (ETSI TS 101388 V1.3.1; 2002-03)
%
% NOTE: Testcables only defined up to 1.1 MHz, thus we need to
% extrapolate for the tone at 1104 kHz!
% Use one of the above models instead!


cable=[];
cable.name='ETSI-ADSL-PE032tab';
cable.model='RLC';
params = [
0.00 409.000 607.640 40.00
 2.50 409.009 607.640 40.00
 10.00 409.140 607.640 40.00
 20.00 409.557 607.640 40.00
 30.00 410.251 607.640 40.00
 40.00 411.216 607.640 40.00
 50.00 412.448 607.640 40.00
 100.00 422.302 607.632 40.00
 150.00 437.339 607.571 40.00
 200.00 456.088 607.328 40.00
 250.00 477.232 606.640 40.00
 300.00 499.762 605.075 40.00
 350.00 522.973 602.047 40.00
 400.00 546.402 596.935 40.00
 450.00 569.755 589.338 40.00
 500.00 592.851 579.377 40.00
 550.00 615.584 567.823 40.00
 600.00 637.894 555.868 40.00
 650.00 659.752 544.658 40.00
 700.00 681.148 534.942 40.00
 750.00 702.083 526.992 40.00
 800.00 722.567 520.732 40.00
 850.00 742.612 515.919 40.00
 900.00 762.236 512.265 40.00
 950.00 781.454 509.503 40.00
 1000.00 800.284 507.415 40.00
 1050.00 818.744 505.831 40.00
 1100.00 836.850 504.623 40.00
];
cable.param.ftmpl=1e3.*params(:,1);
cable.param.Rtmpl=1e-3.*params(:,2);
cable.param.Ltmpl=1e-9.*params(:,3);
cable.param.Ctmpl=1e-12.*params(:,4);
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE04tab';
cable.model='RLC';
params = [
0.00 280.000 587.130 50.00
 2.50 280.007 587.073 50.00
 10.00 280.110 586.736 50.00
 20.00 280.440 586.097 50.00
 30.00 280.988 585.320 50.00
 40.00 281.749 584.441 50.00
 50.00 282.719 583.481 50.00
 100.00 290.437 577.877 50.00
 150.00 302.078 571.524 50.00
 200.00 316.406 564.888 50.00
 250.00 332.365 558.232 50.00
 300.00 349.188 551.714 50.00
 350.00 366.370 545.430 50.00
 400.00 383.590 539.436 50.00
 450.00 400.658 533.759 50.00
 500.00 417.462 528.409 50.00
 550.00 433.941 523.385 50.00
 600.00 450.067 518.677 50.00
 650.00 465.827 514.272 50.00
 700.00 481.224 510.153 50.00
 750.00 496.264 506.304 50.00
 800.00 510.959 502.707 50.00
 850.00 525.324 499.343 50.00
 900.00 539.372 496.197 50.00
 950.00 553.118 493.253 50.00
 1000.00 566.576 490.494 50.00
 1050.00 579.761 487.908 50.00
 1100.00 592.686 485.481 50.00
 ];
cable.param.ftmpl=1e3.*params(:,1);
cable.param.Rtmpl=1e-3.*params(:,2);
cable.param.Ltmpl=1e-9.*params(:,3);
cable.param.Ctmpl=1e-12.*params(:,4);
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE05tab';
cable.model='RLC';
params = [
  0.00 179.000 673.570 50.00
 2.50 179.015 673.462 50.00
 10.00 179.244 672.919 50.00
 20.00 179.970 671.976 50.00
 30.00 181.161 670.892 50.00
 40.00 182.790 669.712 50.00
 50.00 184.823 668.464 50.00
 100.00 199.612 661.674 50.00
 150.00 218.729 654.618 50.00
 200.00 239.143 647.732 50.00
 250.00 259.474 641.206 50.00
 300.00 279.189 635.116 50.00
 350.00 298.120 629.486 50.00
 400.00 316.249 624.307 50.00
 450.00 333.611 619.556 50.00
 500.00 350.264 615.201 50.00
 550.00 366.268 611.210 50.00
 600.00 381.680 607.551 50.00
 650.00 396.553 604.191 50.00
 700.00 410.933 601.103 50.00
 750.00 424.861 598.260 50.00
 800.00 438.376 595.639 50.00
 850.00 451.509 593.216 50.00
 900.00 464.288 590.975 50.00
 950.00 476.740 588.896 50.00
 1000.00 488.888 586.965 50.00
 1050.00 500.752 585.169 50.00
 1100.00 512.350 583.495 50.00
];
cable.param.ftmpl=1e3.*params(:,1);
cable.param.Rtmpl=1e-3.*params(:,2);
cable.param.Ltmpl=1e-9.*params(:,3);
cable.param.Ctmpl=1e-12.*params(:,4);
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE063tab';
cable.model='RLC';
params = [ 
   0.00 113.000 699.260 45.00
 2.50 113.028 697.945 45.00
 10.00 113.441 693.363 45.00
 20.00 114.734 687.010 45.00
 30.00 116.796 680.716 45.00
 40.00 119.512 674.594 45.00
 50.00 122.752 668.691 45.00
 100.00 143.076 642.718 45.00
 150.00 164.880 622.050 45.00
 200.00 185.617 605.495 45.00
 250.00 204.911 592.047 45.00
 300.00 222.866 580.959 45.00
 350.00 239.661 571.689 45.00
 400.00 255.463 563.843 45.00
 450.00 270.414 557.127 45.00
 500.00 284.627 551.321 45.00
 550.00 298.197 546.258 45.00
 600.00 311.200 541.807 45.00
 650.00 323.699 537.866 45.00
 700.00 335.746 534.355 45.00
 750.00 347.387 531.209 45.00
 800.00 358.658 528.375 45.00
 850.00 369.592 525.810 45.00
 900.00 380.217 523.478 45.00
 950.00 390.558 521.349 45.00
 1000.00 400.635 519.399 45.00
 1050.00 410.469 517.606 45.00
 1100.00 420.075 515.953 45.00 
         ];
cable.param.ftmpl=1e3.*params(:,1);
cable.param.Rtmpl=1e-3.*params(:,2);
cable.param.Ltmpl=1e-9.*params(:,3);
cable.param.Ctmpl=1e-12.*params(:,4);
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE09tab';
cable.model='RLC';
params = [
0.00 55.000 750.790 40.00
 2.50 55.088 745.498 40.00
 10.00 56.361 731.955 40.00
 20.00 59.943 716.770 40.00
 30.00 64.780 703.870 40.00
 40.00 70.131 692.703 40.00
 50.00 75.591 682.910 40.00
 100.00 100.779 647.493 40.00
 150.00 121.878 625.138 40.00
 200.00 140.090 609.652 40.00
 250.00 156.290 598.256 40.00
 300.00 171.006 589.504 40.00
 350.00 184.576 582.563 40.00
 400.00 197.229 576.919 40.00
 450.00 209.126 572.237 40.00
 500.00 220.388 568.288 40.00
 550.00 231.106 564.911 40.00
 600.00 241.352 561.989 40.00
 650.00 251.182 559.435 40.00
 700.00 260.643 557.184 40.00
 750.00 269.774 555.184 40.00
 800.00 278.607 553.395 40.00
 850.00 287.169 551.785 40.00
 900.00 295.484 550.328 40.00
 950.00 303.571 549.003 40.00
 1000.00 311.449 547.794 40.00
 1050.00 319.134 546.684 40.00
 1100.00 326.637 545.663 40.00
         ];
cable.param.ftmpl=1e3.*params(:,1);
cable.param.Rtmpl=1e-3.*params(:,2);
cable.param.Ltmpl=1e-9.*params(:,3);
cable.param.Ctmpl=1e-12.*params(:,4);
clist=insertList(clist,cable);

%% ===========================================================================
% ETSI ADSL as models from ETSI 981t08a1

cable=[];
cable.name='ETSI-ADSL-PE032m';
cable.model='BT';

Ro     = 409;
Linf   = 500.00e-6;
Fr     = 270.59e3;
LoLinf = 107.64e-6;
Fm     = 608.77e3;
Nb     = 5.2464;
Cp     = 40e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = (Ro^2/Fr)^2;
cable.param.as  = 0;
cable.param.l0  = LoLinf + Linf;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);


cable=[];
cable.name='ETSI-ADSL-PE04m';
cable.model='BT';

Ro     = 280;
Linf   = 427.12e-6;
Fr     = 251.91e3;
LoLinf = 160.01e-6;
Fm     = 739.05e3;
Nb     = 1.3952;
Cp     = 50e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = (Ro^2/Fr)^2;
cable.param.as  = 0;
cable.param.l0  = LoLinf + Linf;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE05m';
cable.model='BT';

Ro     = 179;
Linf   = 544.25e-6;
Fr     = 135.3e3;
LoLinf = 129.32e-6;
Fm     = 580.92e3;
Nb     = 1.3013;
Cp     = 50e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = (Ro^2/Fr)^2;
cable.param.as  = 0;
cable.param.l0  = LoLinf + Linf;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);


cable=[];
cable.name='ETSI-ADSL-PE063m';
cable.model='BT';

Ro     = 113;
Linf   = 477.42e-6;
Fr     = 79.733e3;
LoLinf = 221.84e-6;
Fm     = 265.70e3;
Nb     = 1.0978;
Cp     = 45e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = (Ro^2/Fr)^2;
cable.param.as  = 0;
cable.param.l0  = LoLinf + Linf;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);

cable=[];
cable.name='ETSI-ADSL-PE09m';
cable.model='BT';

Ro     = 55;
Linf   = 520.45e-6;
Fr     = 31.207e3;
LoLinf = 230.34e-6;
Fm     = 124.04e3;
Nb     = 0.9605;
Cp     = 40e-9;

cable.param.r0c = Ro;    
cable.param.r0s = 0;
cable.param.ac  = (Ro^2/Fr)^2;
cable.param.as  = 0;
cable.param.l0  = LoLinf + Linf;
cable.param.loo = Linf; 
cable.param.b   = Nb;     
cable.param.fm  = Fm;    
cable.param.coo = Cp;
cable.param.c0  = 0;
cable.param.ce  = 0;
cable.param.g0  = 0;
cable.param.ge  = 1;
clist=insertList(clist,cable);
