function tfplist =  etsi_tfplansLegacy(tfplist)
%% ===========================================================================
% etsi_tfplansLegacy(tfplist) - Sets up tfplans for some legacy systems
%                               used for ETSI SpM
%
% Parameter:    tfplist	        Structure describing the time and freq plan
% Returns:      tfplist		Structure describing the time and freq plan
%
% Example(s):
%   ex.tfplist =  etsi_tfplansLegacy(ex.tfplist);   Add to existing tfplist
%   tfplan=getList(ex.tfplist,'ETSI-HDSL-2B1Q-2p'); Get tfplan for HDSL 2B2Q 2p
%
%   ex.tfplist =  etsi_tfplansLegacy([]);           Create a new tfplist
%   tfplan=getList(ex.tfplist,'ETSI-ISDN-4B3T');    Get tfplan for ISDN-4BT3
%
% References:
%   ETSI TM6 034w08
%   ETSI TM6 m01p20a5 and m01p21a7 spm2_LL
%   ETSI STC TM6 TM6(98)10 rev. 5
%% ===========================================================================

%% ===========================================================================
% Copyright (C):                                        
%       2003-2007 by Forschungszentrum Telekommunikation Wien, Austria;
%                                                         All rights reserved.
% Project       : FTW's xDSLsimu
% Author(s)     : Tomas Nordstrom (Tomas.Nordstrom@FTW.at)
%
% CVS:       $Id: etsi_tfplansLegacySpM.m,v 1.2 2005/01/04 10:25:12 tono Exp $
%% ===========================================================================
% Change History
%      2003-11-11 (ToNo) Created (based on fsan_modelsMISC.m)
%% ===========================================================================

%% ===========================================================================
% Use a template, as order is important!

def_tfplan = templateTFP;
def_tfplan.fixBitrate.active = 1; % These systems use fix bitrate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% HDSL 2B1Q model using 1 pair, from m01p20a5
tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-HDSL-2B1Q-1p';
tmp_tfplan.PSD.downstream='modelPSD_HDSL2B1Q(ex.param.frequency.f,1)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 1.16e6]; % Not used
tmp_tfplan.PSD.active.downstream=[1e3 1.16e6];
tmp_tfplan.lcname=tmp_tfplan.name;
tfplist=insertList(tfplist,tmp_tfplan);

% HDSL 2B1Q model using 2 pairs, from m01p20a5
tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-HDSL-2B1Q-2p';
tmp_tfplan.PSD.downstream='modelPSD_HDSL2B1Q(ex.param.frequency.f,2)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 .584e6]; % Not used
tmp_tfplan.PSD.active.downstream=[1e3 .584e6];
tmp_tfplan.lcname=tmp_tfplan.name;
tfplist=insertList(tfplist,tmp_tfplan);

% HDSL 2B1Q model using 3 pairs, from m01p20a5
tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-HDSL-2B1Q-3p';
tmp_tfplan.PSD.downstream='modelPSD_HDSL2B1Q(ex.param.frequency.f,3)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 .392e6]; % Not used
tmp_tfplan.PSD.active.downstream=[1e3 .392e6];
tmp_tfplan.lcname=tmp_tfplan.name;
tfplist=insertList(tfplist,tmp_tfplan);

% HDSL CAP (2-pair) according to TM6(98)10, rev. 5; and m01p20a5
tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-HDSL-CAP-2p';
tmp_tfplan.PSD.downstream='calcPSD([.0001e6 -57 .00398e6 -57 .0215e6 -43 .03902e6 -40 .23758e6 -40 .2551e6 -43 .27262e6 -60 .297e6 -90 1.188e6 -120 30e6 -120],''Log-Linear'',ex.param.frequency.f)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 276.2e3];  % Not used
tmp_tfplan.PSD.active.downstream=[1e3 276.2e3];
tmp_tfplan.lcname=tmp_tfplan.name;
tfplist=insertList(tfplist,tmp_tfplan);

% From 033t17 + the mask above 400e3 (cf. 034w08)
% FIXME: it seems that the point -67 at 1e6 is violating the mask
% and should be at -74

tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-ISDN-4B3T';
tmp_tfplan.PSD.downstream='calcPSD([5000 -40 22500 -36 40000 -37 65000 -40 80000 -43 100000 -50 122500 -62 154500 -60 170000 -61 185000 -65 200000 -69 215000 -74 250000 -82 300000 -78 400000 -74 1e6 -74 4.043e6 -120 30e6 -120 ],''Log-Linear'',ex.param.frequency.f)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 200e3];    % Not used
tmp_tfplan.PSD.active.downstream=[1e3 200e3];
tmp_tfplan.lcname='ISDN-2B1Q'; % FIXME - the true model should be implemented
tfplist=insertList(tfplist,tmp_tfplan);

% ISDN 2B1Q model using 3 pairs, from m01p20a5
tmp_tfplan=def_tfplan;
tmp_tfplan.name='ETSI-ISDN-2B1Q';
tmp_tfplan.PSD.downstream='modelPSD_ISDN2B1Q(ex.param.frequency.f)';
tmp_tfplan.PSD.upstream=tmp_tfplan.PSD.downstream;
tmp_tfplan.PSD.active.upstream=[1e3 160e3];  % Not used
tmp_tfplan.PSD.active.downstream=[1e3 160e3];
tmp_tfplan.lcname=tmp_tfplan.name;
tfplist=insertList(tfplist,tmp_tfplan);
