function tfplist = etsi_tfplansSDSL_RC(tfplist)
%% ===========================================================================
% etsi_tfplansSDSL_RC(tfplist) - Sets up the tfplan for an earlier
%                                definition of ETSI SDSL
%
% Parameter:    tfplist	   Structure describing the time and freq plan
% Returns:      tfplist		Structure describing the time and freq plan
%
% Example(s):
%   ex.tfplist = etsi_tfplanSDSL_RC(ex.tfplist);    Add to existing tfplist
%   tfplan=getList(ex.tfplist,'ETSI-SDSL-RC-sym') ; Get this SDSL plan
%
%   ex.tfplist = etsi_tfplanSDSL_RC([]);            Create a new tfplist
%   tfplan=getList(ex.tfplist,'ETSI-SDSL-RC-sym');  Get this SDSL plan
%
% References:	
%   Draft ETSI TS 101524-2 V1.1.1 (2000-05)
%% ===========================================================================

%% ===========================================================================
% Copyright (C):                                        
%       2000-2007 by Forschungszentrum Telekommunikation Wien, Austria;
%                                                         All rights reserved.
% Project       : FTW's xDSLsimu
% Author(s)     : Tomas Nordstrom (Tomas.B.Nordstrom@Telia.se)
%               : Gernot Schmid (schmid@ftw.at)
%
% CVS:       $Id: etsi_tfplansSDSL_RC.m 686 2007-01-18 15:54:57Z tono $
%% ===========================================================================
% Change History
%      2000-01-19 (GS)   Created
%      2000-09-27 (GS)   Cleaning up for release
%      2005-01-10 (ToNo) Made changes to reflect the new naming convention
%      2005-04-27 (ToNo) Fixed support for Asymmetric SDSL
%% ===========================================================================


%% ===========================================================================
% Use a template, as order is important!

sdsl_tfplan = templateTFP;
    
% SDSL generic default values
sdsl_tfplan.PSD.HAM.active        = 0;
sdsl_tfplan.PSD.active.upstream   = [5e3 2e6];
sdsl_tfplan.PSD.active.downstream = [5e3 2e6];
sdsl_tfplan.fixBitrate.name       = 'SDSL';
sdsl_tfplan.fixBitrate.active     = 1;
sdsl_tfplan.fixBitrate.param      = [];


% SDSL TFPlan definitions
% --------------------------

% Symmetric SDSL as specified in Draft ETSI TS 101524-2 V1.1.1 (2000-05)
basic_tfplan=sdsl_tfplan;
basic_tfplan.name='ETSI-SDSL-RC-sym';
basic_tfplan.lcname='ETSI-SDSL-RC-sym';
basic_tfplan.PSD.downstream ='modelPSD_SDSL_sym_ver1(tfplan.lcname,ex.param.frequency.f)'; 
basic_tfplan.PSD.upstream   = basic_tfplan.PSD.downstream;	
basic_tfplan.PSD.UPBO.method='None'; 
tfplist=insertList(tfplist,basic_tfplan);

%% --------------------
% Set up some basic rates
brates = [256, 512, 1024, 2048, 2304];

for brate = brates
    name = [basic_tfplan.name '-' num2str(brate)];
    fsym=(brate+8)*1e3/3;
    tmp_tfplan= basic_tfplan;
    tmp_tfplan.PSD.active.upstream   = [1 2*fsym];
    tmp_tfplan.PSD.active.downstream = [1 2*fsym];
    tmp_tfplan.name=name;
    tmp_tfplan.lcname=name;
    tfplist=insertList(tfplist,tmp_tfplan);
end;


% Asymmetric SDSL as specified in Draft ETSI TS 101524-2 V1.1.1 (2000-05)
basic_tfplan=sdsl_tfplan;
basic_tfplan.name='ETSI-SDSL-RC-asym';
basic_tfplan.lcname='ETSI-SDSL-RC-asym';
basic_tfplan.PSD.downstream ='modelPSD_SDSL_asym(tfplan.lcname,ex.param.frequency.f,1)'; 
basic_tfplan.PSD.upstream   ='modelPSD_SDSL_asym(tfplan.lcname,ex.param.frequency.f,2)';
basic_tfplan.PSD.UPBO.method='None'; 
tfplist=insertList(tfplist,basic_tfplan);

%% --------------------
% Set up the specific rates
brates = [2048, 2304];

for brate = brates
    name = [basic_tfplan.name '-' num2str(brate)];
    fsym=(brate+8)*1e3/3;
    tmp_tfplan= basic_tfplan;
    tmp_tfplan.PSD.active.upstream   = [1 2*fsym];
    tmp_tfplan.PSD.active.downstream = [1 2.667*fsym];
    tmp_tfplan.name=name;
    tmp_tfplan.lcname=name;
    tfplist=insertList(tfplist,tmp_tfplan);
end;

